@@ -13,8 +13,10 @@ if this_dir not in sys.path:
 
 from common_imports import etree, ElementTree, fileInTestDir, _str, _bytes
 from common_imports import SillyFileLike, LargeFileLike, HelperTestCase
+from common_imports import read_file, write_to_file, BytesIO
 
-class IOTestCaseBase(HelperTestCase):
+
+class _IOTestCaseBase(HelperTestCase):
     """(c)ElementTree compatibility for IO functions/methods
     """
     etree = None
@@ -67,7 +69,7 @@ class IOTestCaseBase(HelperTestCase):
         f = open(self.getTestFilePath('testdump2.xml'), 'rb')
         data2 = f.read()
         f.close()
-        self.assertEquals(data1, data2)
+        self.assertEqual(data1, data2)
 
     def test_tree_io_latin1(self):
         Element = self.etree.Element
@@ -92,7 +94,7 @@ class IOTestCaseBase(HelperTestCase):
         f = open(self.getTestFilePath('testdump2.xml'), 'rb')
         data2 = f.read()
         f.close()
-        self.assertEquals(data1, data2)
+        self.assertEqual(data1, data2)
         
     def test_write_filename(self):
         # (c)ElementTree  supports filename strings as write argument
@@ -100,11 +102,23 @@ class IOTestCaseBase(HelperTestCase):
         handle, filename = tempfile.mkstemp(suffix=".xml")
         self.tree.write(filename)
         try:
-            self.assertEqual(open(filename, 'rb').read().replace(_bytes('\n'), _bytes('')),
+            self.assertEqual(read_file(filename, 'rb').replace(_bytes('\n'), _bytes('')),
                              self.root_str)
         finally:
             os.close(handle)
             os.remove(filename)
+        
+    def test_write_invalid_filename(self):
+        filename = os.path.join(
+            os.path.join('hopefullynonexistingpathname'),
+            'invalid_file.xml')
+        try:
+            self.tree.write(filename)
+        except IOError:
+            pass
+        else:
+            self.assertTrue(
+                False, "writing to an invalid file path should fail")
 
     def test_module_parse_gzipobject(self):
         # (c)ElementTree supports gzip instance as parse argument
@@ -128,7 +142,7 @@ class IOTestCaseBase(HelperTestCase):
         # parse from filename
         
         handle, filename = tempfile.mkstemp(suffix=".xml")
-        open(filename, 'wb').write(self.root_str)
+        write_to_file(filename, self.root_str, 'wb')
         try:
             tree = self.etree.ElementTree()
             root = tree.parse(filename)
@@ -139,19 +153,19 @@ class IOTestCaseBase(HelperTestCase):
 
     def test_class_parse_filename_remove_previous(self):
         handle, filename = tempfile.mkstemp(suffix=".xml")
-        open(filename, 'wb').write(self.root_str)
+        write_to_file(filename, self.root_str, 'wb')
         try:
             tree = self.etree.ElementTree()
             root = tree.parse(filename)
             # and now do it again; previous content should still be there
             root2 = tree.parse(filename)
-            self.assertEquals('a', root.tag)
-            self.assertEquals('a', root2.tag)
+            self.assertEqual('a', root.tag)
+            self.assertEqual('a', root2.tag)
             # now remove all references to root2, and parse again
             del root2
             root3 = tree.parse(filename)
-            self.assertEquals('a', root.tag)
-            self.assertEquals('a', root3.tag)
+            self.assertEqual('a', root.tag)
+            self.assertEqual('a', root3.tag)
             # root2's memory should've been freed here
             # XXX how to check?
         finally:
@@ -165,8 +179,8 @@ class IOTestCaseBase(HelperTestCase):
         # parse from file object
         
         handle, filename = tempfile.mkstemp(suffix=".xml")
-        os.write(handle, self.root_str)
         try:
+            os.write(handle, self.root_str)
             f = open(filename, 'rb')
             tree = self.etree.ElementTree()
             root = tree.parse(f)
@@ -183,14 +197,14 @@ class IOTestCaseBase(HelperTestCase):
         # parse from unamed file object    
         f = SillyFileLike()
         root = self.etree.ElementTree().parse(f)
-        self.assert_(root.tag.endswith('foo'))
+        self.assertTrue(root.tag.endswith('foo'))
 
     def test_module_parse_large_fileobject(self):
         # parse from unamed file object
         f = LargeFileLike()
         tree = self.etree.parse(f)
         root = tree.getroot()
-        self.assert_(root.tag.endswith('root'))
+        self.assertTrue(root.tag.endswith('root'))
 
     def test_module_parse_fileobject_error(self):
         class LocalError(Exception):
@@ -224,23 +238,125 @@ class IOTestCaseBase(HelperTestCase):
                         raise LocalError
         f = TestFile()
         self.assertRaises(LocalError, self.etree.parse, f)
-        self.assertEquals(f.counter, len(f.data)+1)
+        self.assertEqual(f.counter, len(f.data)+1)
 
     def test_module_parse_fileobject_type_error(self):
         class TestFile:
             def read(*args):
                 return 1
         f = TestFile()
-        self.assertRaises(TypeError, self.etree.parse, f)
 
-    
-class ETreeIOTestCase(IOTestCaseBase):
+        try:
+            expect_exc = (TypeError, self.etree.ParseError)
+        except AttributeError:
+            expect_exc = TypeError
+        self.assertRaises(expect_exc, self.etree.parse, f)
+
+    def test_etree_parse_io_error(self):
+        # this is a directory name that contains characters beyond latin-1
+        dirnameEN = _str('Directory')
+        dirnameRU = _str('ÐšÐ°Ñ‚Ð°Ð»Ð¾Ð³')
+        filename = _str('nosuchfile.xml')
+        dn = tempfile.mkdtemp(prefix=dirnameEN)
+        try:
+            self.assertRaises(IOError, self.etree.parse, os.path.join(dn, filename))
+        finally:
+            os.rmdir(dn)
+        dn = tempfile.mkdtemp(prefix=dirnameRU)
+        try:
+            self.assertRaises(IOError, self.etree.parse, os.path.join(dn, filename))
+        finally:
+            os.rmdir(dn)
+
+    def test_parse_utf8_bom(self):
+        utext = _str('Søk på nettet')
+        uxml = '<?xml version="1.0" encoding="UTF-8"?><p>%s</p>' % utext
+        bom = _bytes('\\xEF\\xBB\\xBF').decode(
+            "unicode_escape").encode("latin1")
+        self.assertEqual(3, len(bom))
+        f = tempfile.NamedTemporaryFile()
+        try:
+            f.write(bom)
+            f.write(uxml.encode("utf-8"))
+            f.flush()
+            tree = self.etree.parse(f.name)
+        finally:
+            f.close()
+        self.assertEqual(utext, tree.getroot().text)
+
+    def test_iterparse_utf8_bom(self):
+        utext = _str('Søk på nettet')
+        uxml = '<?xml version="1.0" encoding="UTF-8"?><p>%s</p>' % utext
+        bom = _bytes('\\xEF\\xBB\\xBF').decode(
+            "unicode_escape").encode("latin1")
+        self.assertEqual(3, len(bom))
+        f = tempfile.NamedTemporaryFile()
+        try:
+            f.write(bom)
+            f.write(uxml.encode("utf-8"))
+            f.flush()
+            elements = [el for _, el in self.etree.iterparse(f.name)]
+            self.assertEqual(1, len(elements))
+            root = elements[0]
+        finally:
+            f.close()
+        self.assertEqual(utext, root.text)
+
+    def test_iterparse_utf16_bom(self):
+        utext = _str('Søk på nettet')
+        uxml = '<?xml version="1.0" encoding="UTF-16"?><p>%s</p>' % utext
+        boms = _bytes('\\xFE\\xFF \\xFF\\xFE').decode(
+            "unicode_escape").encode("latin1")
+        self.assertEqual(5, len(boms))
+        xml = uxml.encode("utf-16")
+        self.assertTrue(xml[:2] in boms, repr(xml[:2]))
+
+        f = tempfile.NamedTemporaryFile()
+        try:
+            f.write(xml)
+            f.flush()
+            elements = [el for _, el in self.etree.iterparse(f.name)]
+            self.assertEqual(1, len(elements))
+            root = elements[0]
+        finally:
+            f.close()
+        self.assertEqual(utext, root.text)
+
+
+class ETreeIOTestCase(_IOTestCaseBase):
     etree = etree
-    
+
+    def test_write_compressed_text(self):
+        Element = self.etree.Element
+        SubElement = self.etree.SubElement
+        ElementTree = self.etree.ElementTree
+        text = _str("qwrtioüöä")
+
+        root = Element('root')
+        root.text = text
+        child = SubElement(root, 'sub')
+        child.text = 'TEXT'
+        child.tail = 'TAIL'
+        SubElement(root, 'sub').text = text
+
+        tree = ElementTree(root)
+        out = BytesIO()
+        tree.write(out, method='text', encoding='utf8', compression=9)
+        out.seek(0)
+
+        f = gzip.GzipFile(fileobj=out)
+        try:
+            result = f.read().decode('utf8')
+        finally:
+            f.close()
+        self.assertEqual(text+'TEXTTAIL'+text, result)
+
+
 if ElementTree:
-    class ElementTreeIOTestCase(IOTestCaseBase):
+    class ElementTreeIOTestCase(_IOTestCaseBase):
         etree = ElementTree
 
+
 def test_suite():
     suite = unittest.TestSuite()
     suite.addTests([unittest.makeSuite(ETreeIOTestCase)])
@@ -248,5 +364,6 @@ def test_suite():
         suite.addTests([unittest.makeSuite(ElementTreeIOTestCase)])
     return suite
 
+
 if __name__ == '__main__':
     print('to test use test.py %s' % __file__)
